@@ -243,7 +243,7 @@ void KCMColors::installSchemeFile(const QString &path)
 
     m_model->load();
 
-    const auto results = m_model->match(m_model->index(0, 0), ColorsModel::SchemeNameRole, newName, 1, Qt::MatchExactly);
+    const auto results = m_model->match(m_model->index(0, 0), ColorsModel::SchemeNameRole, newName);
     if (!results.isEmpty()) {
         m_model->setSelectedScheme(newName);
     }
@@ -348,10 +348,13 @@ void KCMColors::load()
 
 void KCMColors::save()
 {
-    ManagedConfigModule::save();
+    // We need to save the colors change first, to avoid a situation,
+    // when we announced that the color scheme has changed, but
+    // the colors themselves in the color scheme have not yet
     if (m_selectedSchemeDirty || m_activeSchemeEdited) {
         saveColors();
     }
+    ManagedConfigModule::save();
     m_activeSchemeEdited = false;
 
     processPendingDeletions();
@@ -453,9 +456,7 @@ void KCMColors::saveColors()
 
     m_config->sync();
 
-    runRdb(KRdbExportQtColors | KRdbExportGtkTheme | KRdbExportGtkColors | (m_applyToAlien ? KRdbExportColors : 0));
-
-    saveGtkColors(config);
+    runRdb(KRdbExportQtColors | KRdbExportGtkTheme | (m_applyToAlien ? KRdbExportColors : 0));
 
     QDBusMessage message = QDBusMessage::createSignal(QStringLiteral("/KGlobalSettings"),
                                                       QStringLiteral("org.kde.KGlobalSettings"),
