@@ -76,9 +76,7 @@ int set_repeat_mode(TriState keyboardRepeatMode)
 
 void init_keyboard_hardware()
 {
-	auto inputConfig = KSharedConfig::openConfig( QStringLiteral("kcminputrc") );
-	inputConfig->reparseConfiguration();
-	KConfigGroup config(inputConfig, "Keyboard");
+    KConfigGroup config(KSharedConfig::openConfig( QStringLiteral("kcminputrc") ), "Keyboard");
 
 	QString keyRepeatStr = config.readEntry("KeyboardRepeating", TriStateHelper::getString(STATE_ON));
 	TriState keyRepeat = STATE_UNCHANGED;
@@ -102,5 +100,4 @@ void init_keyboard_hardware()
         KModifierKeyInfo keyInfo;
         keyInfo.setKeyLocked(Qt::Key_NumLock, numlockState == STATE_ON);
 	}
-	XFlush(QX11Info::display());
 }
